@@ -118,9 +118,12 @@
 # * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
 # *                                                                         *
 # ***************************************************************************/
+import sys
+
+if sys.version_info[0] != 2:
+    sys.exit("Sorry, Zenmap requires Python 2")
 
 import errno
-import sys
 import os
 import os.path
 import re
@@ -381,15 +384,15 @@ for dir in dirs:
                 continue
 
             if os.path.isdir(output):
-                os.chmod(output, S_IRWXU | \
-                                 S_IRGRP | \
-                                 S_IXGRP | \
-                                 S_IROTH | \
+                os.chmod(output, S_IRWXU |
+                                 S_IRGRP |
+                                 S_IXGRP |
+                                 S_IROTH |
                                  S_IXOTH)
             else:
-                os.chmod(output, S_IRUSR | \
-                                 S_IWUSR | \
-                                 S_IRGRP | \
+                os.chmod(output, S_IRUSR |
+                                 S_IWUSR |
+                                 S_IRGRP |
                                  S_IROTH)
 
     def fix_paths(self):
